#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Time    : 2023/5/11 19:12
@Author  : alexanderwu
@File    : test_project_management.py
"""

from metagpt.actions.project_management import WriteTasks, AssignTasks


class TestCreateProjectPlan:
    pass


class TestAssignTasks:
    pass
